@@ -28,6 +28,7 @@ from blessings import Terminal
 from docutils.utils import Reporter
 from enum import Enum
 from invoke import Collection, task, Exit
+from pip import __version__ as pip_version
 import readme_renderer.rst  # transitively required via twine in setup.py
 from releases.util import parse_changelog
 from tabulate import tabulate
@@ -804,7 +805,7 @@ def publish(
 @task
 def test_install(c, directory):
     """
-    Test installation of previously built artifacts found in ``directory``.
+    Test installation of build artifacts found in ``$directory/dist``.
 
     Uses the `venv` module to build temporary virtualenvs.
     """
@@ -822,14 +823,19 @@ def test_install(c, directory):
         if "py2" in archive and "py3" not in archive:
             continue
         with tmpdir() as tmp:
+            # Make temp venv
             builder.create(tmp)
-            # Does it install cleanly?
+            # Obligatory: make inner pip match outer pip (version obtained from
+            # this file's executable env, up in import land); very frequently
+            # venv-made envs have a bundled, older pip :(
+            pip = os.path.join(tmp, "bin", "pip")
+            c.run("{} install pip=={}".format(pip, pip_version))
+            # Does the package under test install cleanly?
             # TODO: might be nice to have a further 'can you import whatever it
             # was' test
-            # TODO: obligatory "is it worth upgrading pip always?"
             c.run(
                 "{} install --disable-pip-version-check {}".format(
-                    os.path.join(tmp, "bin", "pip"), archive
+                    pip, archive
                 )
             )
 
@@ -846,6 +852,7 @@ def get_archives(directory):
     )
 
 
+@task
 def upload(c, directory, index=None, sign=False, dry_run=False):
     """
     Upload (potentially also signing) all artifacts in ``directory/dist``.
@@ -881,9 +888,9 @@ def upload(c, directory, index=None, sign=False, dry_run=False):
                 "installed to GPG-sign!"
             )
         for archive in archives:
-            cmd = "{} --detach-sign -a --passphrase-fd 0 {{}}".format(
+            cmd = "{} --detach-sign --armor --passphrase-fd=0 --batch --pinentry-mode=loopback {{}}".format(  # noqa
                 gpg_bin
-            )  # noqa
+            )
             c.run(cmd.format(archive), in_stream=input_, dry=dry_run)
             input_.seek(0)  # So it can be replayed by subsequent iterations
     # Upload
@@ -943,6 +950,7 @@ ns = Collection(
     push,
     test_install,
     tidelift,
+    upload,
 )
 # Hide stdout by default, preferring to explicitly enable it when necessary.
 ns.configure({"run": {"hide": "stdout"}})
