@@ -9,7 +9,7 @@ from invoke.vendor.six import PY2
 from invoke.vendor.lexicon import Lexicon
 from invoke import MockContext, Result, Config, Exit
 from docutils.utils import Reporter
-from mock import Mock, patch
+from mock import Mock, patch, call
 import pytest
 from pytest import skip
 from pytest_relaxed import trap, raises
@@ -33,6 +33,7 @@ from invocations.packaging.release import (
     publish,
     status,
     upload,
+    test_install,
     ns as release_ns,
 )
 
@@ -181,9 +182,18 @@ class load_version_:
 
     @patch("invocations.packaging.release.sys.modules", wraps=sys.modules)
     def reloads_version_in_case_edited_during_run(self, modules):
-        # Necessary as default mocks don't mock dunder-attrs
-        modules.__getitem__.return_value._version = Mock(__version__="1.0.0")
+        # NOTE: mock doesn't mock/wrap dunder-attrs well (eg see python core
+        # bug #25597) so we gotta rub some more on top, esp for eg
+        # Python 3.8+ importlib which does additional setattrs and pops.
+        # (but we still wraps= in @patch as it smooths over other bits we don't
+        # care about mocking, at least under Python <3.8)
+        even_faker_package = Mock(_version=Mock(__version__="1.0.0"))
+        modules.__getitem__.return_value = even_faker_package
+        modules.get.return_value = even_faker_package
         self._expect_version("1.0.0")
+        # Expect our own internal pops (the stdlib ones, eg under 3.8+, don't
+        # exactly match these - no 2nd arg - so we can be pretty sure this
+        # won't incorrectly pass due to them)
         modules.pop.assert_any_call("fakepackage._version", None)
         modules.pop.assert_any_call("fakepackage", None)
 
@@ -1003,7 +1013,7 @@ class build_:
 
 
 class upload_:
-    def _check_upload(self, c, kwargs=None, flags=None):
+    def _check_upload(self, c, kwargs=None, flags=None, extra=None):
         """
         Expect/call upload() with common environment and settings/mocks.
 
@@ -1026,6 +1036,8 @@ class upload_:
         if flags:
             cmd += " {}".format(flags)
         cmd += " {}".format(self.files)
+        if extra:
+            cmd += " {}".format(extra)
         return cmd
 
     def twine_uploads_dist_contents_with_wheels_first(self):
@@ -1046,9 +1058,26 @@ class upload_:
         print.assert_any_call("Would publish via: {}".format(cmd))
         c.run.assert_called_once_with("ls -l {}".format(self.files))
 
-    def allows_signing_via_gpg(self):
-        # Kind of a pain to test :(
-        skip()
+    @patch("invocations.packaging.release.getpass.getpass")
+    def allows_signing_via_gpg(self, getpass):
+        c = MockContext(run=True, repeat=True)
+        getpass.return_value = "super sekrit"
+        twine_upload = self._check_upload(
+            c, kwargs=dict(sign=True), extra="somedir/dist/*.asc"
+        )
+        calls = c.run.mock_calls
+        # Looked for gpg
+        assert calls[0] == call("which gpg", hide=True, warn=True)
+        # Signed wheel
+        flags = "--detach-sign --armor --passphrase-fd=0 --batch --pinentry-mode=loopback"  # noqa
+        template = "gpg {} somedir/dist/foo.{{}}".format(flags)
+        assert calls[1][1][0] == template.format("whl")
+        # Spot check: did use in_stream to submit passphrase
+        assert "in_stream" in calls[1][2]
+        # Signed tgz
+        assert calls[2][1][0] == template.format("tar.gz")
+        # Uploaded (and w/ asc's)
+        c.run.assert_any_call(twine_upload)
 
 
 class Kaboom(Exception):
@@ -1183,14 +1212,39 @@ class publish_:
 
 
 class test_install_:
-    def pip_installs_into_venv(self):
-        skip()
-
-    def aborts_on_python2(self):
-        skip()
-
-    def skips_non_universal_python2_wheels(self):
-        skip()
+    @patch("invocations.packaging.release.pip_version", "lmao")
+    @patch("invocations.packaging.release.get_archives")
+    @patch("invocations.util.mkdtemp")
+    @patch("invocations.util.rmtree", Mock("rmtree"))  # Just a neuter
+    @patch("venv.EnvBuilder")
+    def installs_all_archives_in_fresh_venv_with_matching_pip(
+        self, builder, mkdtemp, get_archives
+    ):
+        # Setup & run
+        c = MockContext(run=True, repeat=True)
+        mkdtemp.return_value = "tmpdir"
+        get_archives.return_value = ["foo.tgz", "foo.whl"]
+        test_install(c, directory="whatever")
+        # Create factory
+        builder.assert_called_once_with(with_pip=True)
+        # Used helper to get artifacts
+        get_archives.assert_called_once_with("whatever")
+        # venv factory ran twice in some temp dir
+        builder.return_value.create.assert_has_calls(
+            [call("tmpdir"), call("tmpdir")]
+        )
+        pip_base = "tmpdir/bin/pip install --disable-pip-version-check"
+        c.run.assert_has_calls(
+            [
+                # Pip installed to same version as running interpreter's pip
+                call("tmpdir/bin/pip install pip==lmao"),
+                # Archives installed into venv
+                call("{} foo.tgz".format(pip_base)),
+                # And repeat
+                call("tmpdir/bin/pip install pip==lmao"),
+                call("{} foo.whl".format(pip_base)),
+            ]
+        )
 
 
 class push_:
@@ -1201,6 +1255,7 @@ class push_:
         c.run.assert_called_once_with("git push --follow-tags --no-verify")
 
     @trap
+    @patch("invocations.environment.os.environ", dict(CIRCLECI=""))
     def honors_dry_run(self):
         c = MockContext(run=True)
         push(c, dry_run=True)
@@ -1279,6 +1334,7 @@ class namespace:
            status
            test-install
            tidelift
+           upload
         """.split()
         assert set(release_ns.task_names) == set(names)
 
